/*################################################################################
# Linux Management Providers (LMP), Simple Identity Management provider package
# Copyright (C) 2007 Frederic Desmons, ETRI <desmons@etri.re.kr, desmons_frederic@yahoo.fr>
# 
# This program is being developed under the "OpenDRIM" project.
# The "OpenDRIM" project web page: http://opendrim.sourceforge.net
# The "OpenDRIM" project mailing list: opendrim@googlegroups.com
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 2
# of the License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#################################################################################

#################################################################################
# To contributors, please leave your contact information in this section
# AND comment your changes in the source code.
# 
# Modified by 2009 Khahramon NURIDDINOV, TUIT <qahramon0786@gmail.com>
# Modified by 2009 Guillaume BOTTEX, ETRI <guillaumebottex@etri.re.kr>
################################################################################*/

#include "OpenDRIM_AccountManagementCapabilitiesAccess.h"

string InstanceID;

int Simple_Identity_Management_OpenDRIM_AccountManagementCapabilities_load(const CMPIBroker* broker, string& errorMessage) {
	_E_;
	CF_assert(CF_getSystemName(InstanceID, errorMessage));
	_L_;
	return OK;
}

int Simple_Identity_Management_OpenDRIM_AccountManagementCapabilities_unload(string& errorMessage) {
	_E_;
	// TODO
	_L_;
	return OK;
}

int Simple_Identity_Management_OpenDRIM_AccountManagementCapabilities_retrieve(const CMPIBroker* broker, const CMPIContext* ctx, vector<OpenDRIM_AccountManagementCapabilities>& result, const char** properties, string& errorMessage, const string& discriminant) {
	_E_;
	// Only one instance
	// The key properties are based on the system name
	OpenDRIM_AccountManagementCapabilities instance;
	instance.setInstanceID(InstanceID);
	
	if(discriminant=="ei")
		CF_assert(Simple_Identity_Management_OpenDRIM_AccountManagementCapabilities_populate(instance, errorMessage));
	
	result.push_back(instance);
	_L_;
	return OK;
}

int Simple_Identity_Management_OpenDRIM_AccountManagementCapabilities_getInstance(const CMPIBroker* broker, const CMPIContext* ctx, OpenDRIM_AccountManagementCapabilities& instance, const char** properties, string& errorMessage) {
	_E_;
	// Only one instance
	// The key properties are based on the system name
	if (instance.InstanceID!=InstanceID)
	{
		errorMessage = "No instance";
		return NOT_FOUND;
	}
	
	CF_assert(Simple_Identity_Management_OpenDRIM_AccountManagementCapabilities_populate(instance, errorMessage));
	_L_;
	return OK;
}

int Simple_Identity_Management_OpenDRIM_AccountManagementCapabilities_setInstance(const CMPIBroker* broker, const CMPIContext* ctx, const OpenDRIM_AccountManagementCapabilities& newInstance, const OpenDRIM_AccountManagementCapabilities& oldInstance, const char** properties, string& errorMessage) {
	_E_;
	// TODO
	_L_;
	return NOT_SUPPORTED;
}

int Simple_Identity_Management_OpenDRIM_AccountManagementCapabilities_createInstance(const CMPIBroker* broker, const CMPIContext* ctx, const OpenDRIM_AccountManagementCapabilities& instance, string& errorMessage) {
	_E_;
	// TODO
	_L_;
	return NOT_SUPPORTED;
}

int Simple_Identity_Management_OpenDRIM_AccountManagementCapabilities_deleteInstance(const CMPIBroker* broker, const CMPIContext* ctx, const OpenDRIM_AccountManagementCapabilities& instance, string& errorMessage) {
	_E_;
	// TODO
	_L_;
	return NOT_SUPPORTED;
}

int Simple_Identity_Management_OpenDRIM_AccountManagementCapabilities_CreateGoalSettings(const CMPIBroker* broker, const CMPIContext* ctx, const OpenDRIM_AccountManagementCapabilities& instance, unsigned short& returnValue, const OpenDRIM_AccountManagementCapabilities_CreateGoalSettings_In& in, OpenDRIM_AccountManagementCapabilities_CreateGoalSettings_Out& out, string& errorMessage) {
	_E_;
	// TODO
	_L_;
	return OK;
}

int Simple_Identity_Management_OpenDRIM_AccountManagementCapabilities_populate(OpenDRIM_AccountManagementCapabilities& instance, string& errorMessage) {
	_E_;
	instance.setElementName("Account Management Capabilities");	
	instance.setMaxElementNameLen(256);
	instance.setElementNameEditSupported(true);
	_L_;
	return OK;
}

